#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>

#include "linked_list.h"


void print_list(llist_t *list) {
  if (list == NULL) {
    return;
  }

  node_t *node = list->head;
  while (node) {
    printf("Current node at %p with value %d. Next node %p\n",
           node, node->data, node->next);
    node = node->next;
  }
}

int add_front(llist_t *list, node_t *node) {
  // Error checking
  if (node == NULL || list == NULL) {
    return -2;
  }
  if (node->next != NULL) {
    return -1;
  }

  node_t *oldhead = list->head;
  list->head = node;
  node->next = oldhead;
  
  list->node_count++;
  return 0;
}

int add_back(llist_t *list, node_t *node) {
  // Error checking
  if (node == NULL || list == NULL) {
    return -2;
  }
  if (node->next != NULL) {
    return -1;
  }

  node_t *node_iter = list->head;

  if (node_iter == NULL) {
    // The list is empty, the front and back are the same
    list->head = node;
    list->node_count++;
    return 0;
  }

  // Iterate until the end
  while(node_iter->next != NULL) {
    node_iter = node_iter->next;
  }

  // node is now the last node in the list
  node_iter->next = node;

  list->node_count++;
  return 0;
}

node_t *peek_front(llist_t *list) {
  if (list == NULL) {
    return NULL;
  }

  return list->head;
}

int pop_front(llist_t *list) {
  if (list == NULL) {
    return -2;
  }
  if (list->head == NULL) {
    return -1;
  }

  node_t *oldhead = list->head;
  list->head = list->head->next;

  free(oldhead);
  return 0;
}

int list_length(llist_t *list) {
  return list->node_count;
}
