

/* This file contains the definitions of nodes and linked lists
 * and the declarations of functions that act on nodes linked lists
 */

/* This is the type for the nodes of the linked list */
typedef struct Node {
  struct Node *next;
  int data;
} node_t;


/* This is the type for the linked list.
 * Although not totally requires, we keep a node_count 
 * to make some implementation easier */
typedef struct {
  node_t *head;
  int node_count;
} llist_t;

/** Walk the list and print each node and its data
 * @param list the list to print
 */
void print_list(llist_t *list);

/** Adds a node to the front of the linked list 
 * @param list the linked list to add the node to
 * @param node the node to add to the linked list
 * @return 0 for success, error otherwise
 *    -1 means node is not initialized correctly
 *    -2 means node or list was null
 */
int add_front(llist_t *list, node_t *node);

/** Adds a node to the end/back of the linked list 
 * @param list the linked list to add the node to
 * @param node the node to add to the linked list
 * @return 0 for success, error otherwise
 *    -1 means node is not initialized correctly
 *    -2 means node or list was null
 */
int add_back(llist_t *list, node_t *node);

/** Peeks at the front of the list
 * @param the list to get the front element of
 * @return the frontmost node of the list, or null if the list is empty
 */
node_t* peek_front(llist_t *list);

/** Removes the first element from the list
 * @param the list to act on
 * @return 0 if success, error otherwise
 *    -1 means empty list
 *    -2 means null list
 */
int pop_front(llist_t *list);

/** Returns the length of the linked list 
 * @param list the list to get the size of
 * @return the size/length of the list
 */
int list_length(llist_t *list);

